@@ -3,20 +3,10 @@
 #include <unistd.h>
 #include <sys/types.h>
 
-char *test_Unixy_getcwd()
-{
-    bstring dir = Unixy_getcwd();
-    mu_assert(dir != NULL, "getcwd failed.");
-    debug("CWD is: %s", bdata(dir));
-
-    bdestroy(dir);
-    return NULL;
-}
 
 char *test_Unixy_chroot_fails()
 {
-    bstring dir = Unixy_getcwd();
-    mu_assert(dir != NULL, "can't getcwd in chroot test.");
+    bstring dir = bfromcstr(".");
 
     int rc = Unixy_chroot(dir);
     mu_assert(rc == -1, "We shouldn't be able to chroot unless running as root.");
@@ -28,13 +18,12 @@ char *test_Unixy_chroot_fails()
 
 char *test_Unixy_drop_priv_fails()
 {
-    bstring dir = Unixy_getcwd();
-    mu_assert(dir != NULL, "can't getcwd in chroot test.");
 
+    bstring dir = bfromcstr(".");
     Unixy_drop_priv(dir);
     // the results of this are so variable we can't check it
-    
     bdestroy(dir);
+    
     return NULL;
 }
 
@@ -61,7 +50,6 @@ char *test_Unixy_pid_file()
 char * all_tests() {
     mu_suite_start();
 
-    mu_run_test(test_Unixy_getcwd);
     if(getuid() != 0) {
         mu_run_test(test_Unixy_chroot_fails);
         mu_run_test(test_Unixy_drop_priv_fails);
